﻿Function Format-CPPMEndpoints{
<# 
    .Synopsis
    Funksjon som formaterer en liste med enheter fra clearpass-apiet, til en liste med objekter med parametrene enhetene hadde i uthentingen, med prefixen CPPM_

    .Description
    Funksjonen får inn et objekt som inneholder enhetene som ble hentet fra Get-CPPMEndpoints. Funksjonen går deretter gjennom hver enhet, lagrer attributter med prefix i et nytt objekt, og legger det
    nye objektet inn i et nytt array. Når alle enheter har blitt gjort om til objekter i det nye arrayet, returneres arrayet.

    .Parameter CPPMData
    Spesifiserer arrayet med enheter som skal formateres med prefix.
    
    .Example
    PS>$auth = Get-CPPMApiAccessToken -CPPMURL "https://cppmserver.domain.local:443" -clientID "apiClient1" -clientSecret "HevfFpbkeRvd44dv3R6j21Dvvrzpf"
    PS>$CPPMRawData = Get-CPPMEndPoints -bearerToken $auth.access_token -CPPMURL "https://cppmserver.domain.local:443"
    PS>$CPPMFormatertData = Format-CPPMEndpoints -CPPMData $CPPMRawData._embedded.items

    Legger det formaterte arrayet med enhetene inn i $CPPMFormatertData, klar til bruk videre av Merge-Enhetstabell

    .Notes
    Funksjonen skal formatere det som ble hentet fra Clearpass, slik at jobben for Merge-Enhetstabell blir enklere.

#>

    [CmdletBinding()]
    param(
        [Parameter(Mandatory, ValueFromPipeline)][object[]]$CPPMData
    )
    BEGIN {
        Write-Verbose "Kjører Format-CPPMEndpoints"

        #Initialiserer arrayet, og setter prefixen som skal gå foran alle informasjonspunkter som lagres
        $formatertData = @()
        $PropertyPrefix = "CPPM_"
        [DateTime]$now = [DateTime]::Now

        $IP_ARRAY = @{}
        $IP_ARRAY = Format-IPtable -path ($PSScriptRoot + '\..\DHCP scopes, Domstoladministrasjonen.xlsx')

        #Løkke som går gjennom hvert endpoint i inndaten, formaterer punktene, og legger de inn i formatertData-arrayet
        Write-Verbose "Starter løkke for gjennomgang av inndata"
    }#/BEGIN

    PROCESS{
        ForEach($endpoint in $CPPMData){
            #Initialiserer objektet som skal inneholde informasjonen
            $mac = $endpoint.mac
            Write-Verbose "`tStarter gjennomgang for enhet $mac"
            $tempObject = New-Object PSCustomObject
            
            #Lager og formaterer IPLok
            $ip = $endpoint.ip
            $ipcomp = Format-IPLok $ip

            if($IP_ARRAY.containskey($ipcomp)){
                $iplok = $IP_ARRAY.$ipcomp
            }else{
                if($ip -like "*::*:*:*:*"){
                    $IPlok += "Unknown Local IPv6"
                    Write-Debug "$ip er en Unknown Local IPv6"
                }elseif ($ip -like "*.*.*.*") {
                    $IPlok += "Unknown IPv4"
                    Write-Debug "$ip er en Unknown IPv4"
                }elseif ($ip -like "????:????:*"){
                    $IPlok += "Unknown Public IPv6"
                    Write-Debug "$ip er en Unknown Public IPv6"
                }
            }
            $tempobject | Add-Member -MemberType NoteProperty -Name CPPM_IPLOK -Value $iplok


            #Formaterer MAC-adresse med kolon mellom hvert andre tegn, og uppercase bokstaver
            $CPPM_MAC = $endpoint.mac | Format-MAC
    
            #Parser updated_at parameteret til et datetime-objekt, slik at datoformatet blir likt som de andre datakildene
            $updateTime = [datetime]::ParseExact($endpoint.updated_at, 'yyyy-MM-ddTHH:mm:sszzz',[Globalization.CultureInfo]::InvariantCulture)

            Write-Debug "`t`t`tLegger inn property mac = $CPPM_MAC"
            $tempObject | Add-Member -NotePropertyName $PropertyPrefix"MAC" -NotePropertyValue $CPPM_MAC

            Write-Debug "`t`t`tLegger inn property updated_at = $updateTime"
            $tempObject | Add-Member -NotePropertyName $propertyPrefix"Updated_At" -NotePropertyValue $updateTime

            #Løkke som legger inn resten av paremeterne som ikke trenger mere formatering enn prefix.
            ForEach($property in $endpoint.psobject.Properties.name){
                If(-not(($property -eq "mac") -or ($property -eq "updated_at"))){
                    $propName = (Get-Culture).TextInfo.ToTitleCase($property)
                    $propVal = $endpoint.$property
                    Write-Debug "`t`t`tLegger til property $propName = $propVal"
                    $tempObject | Add-Member -NotePropertyName $PropertyPrefix$propName -NotePropertyValue $endpoint.$property
                }#/If
            }#/ForEach Property in endpoint.psobject.Properties.name

            $tempObject | Add-Member -NotePropertyName $PropertyPrefix"Dato" -NotePropertyValue $now

            Write-Verbose "`tSender enhet $CPPM_MAC til formatertData"
            $formatertData += $tempObject
        }#/ForEach Endpoint in CPPMData
    }#/PROCESS

    END{
        Write-Verbose "Returnerer formatertData`n"
        $formatertData
    }#/END
}
