﻿Function Get-CPPMEndpoints {
<# 
    .Synopsis
    Funksjon for å hente ut enheter som er funnet av clearpass, i et definert tidsrom, fra Clearpass Insight sin API.

    .Description
    Funksjon som bruker access-token fra Get-CPPMApiAccessToken for å hente ut ClearPass Insight enhetene som er funnet i et tidsrom, som ligger i clearpass. Funksjonen returnerer et objekt, der $objekt._embedded.items er en liste over enheter.
    Funksjonen kjører API-kallet /api/insight/endpoint

    .Parameter bearerToken
    Spesifiserer access-token for å få tilgang til APIet. Vanligvis hentet fra Get-CPPMApiAccessToken

    .Parameter CPPMURL
    Spesifiserer URL til Clearpass-serveren

    .Parameter startTime
    Spesifiserer unix-timestamp for starttidspunktet for uthentingen.

    .Example
    PS>$CPPMRawData = Get-CPPMEndpoints -bearerToken "AbCdEf123456" -CPPMURL "https://cppmserver.domain.local:443" -startTime 0
    Henter alle enheter funnet fra 1/1/1970.

    .Example
    PS>$24hAgo = (([DateTimeOffset](Get-Date)).ToUnixTimeSeconds())-86400
    PS>$CPPMRawData = Get-CPPMEndPoints -bearerToken "AbCdEf123456" -CPPMURL "https://cppmserver.domain.local:443" -startTime $24hAgo
    Henter alle enheter funnet i løpet av de siste 24 timer.

    .Example
    PS>$auth = Get-CPPMApiAccessToken -CPPMURL "https://cppmserver.domain.local:443" -clientID "apiClient1" -clientSecret "HevfFpbkeRvd44dv3R6j21Dvvrzpf"
    PS>$CPPMRawData = Get-CPPMEndPoints -bearerToken $auth.access_token -CPPMURL "https://cppmserver.domain.local:443"
    PS>$CCPMFormatertData = Format-CPPMEndpoints -CPPMData $CPPMRawData._embedded.items
    Henter access-token med Get-CPPMApiAccessToken, bruker denne for å hente endpoints med denne funksjonen, og sender det videre til formateringsfunksjonen.

    .Notes
    Funksjonen er ment å brukes i kombinasjon med Get-CPPMApiAccesToken og Format-CPPMEndpoints. Enhetene som er hentet ligger i outputtens ._embedded.items

#>

[CmdletBinding()]
Param(
    [Parameter(Mandatory, ValueFromPipeline)][string]$bearerToken,
    [Parameter(Mandatory)][string]$CPPMURL,
    [Parameter()][int]$startTime=-1
)
    
    #Finner nåtid i unix-sekunder
    $unixTime = ([DateTimeOffset](Get-Date)).ToUnixTimeSeconds()

    #Validerer startTime-input. Om starttiden er satt til mindre enn 0 eller mere enn nå, defaulter den til å hente info fra 24-timer tilbake
    if(($startTime -lt 0) -or ($startTime -ge $unixTime)){
        $startTime = $unixTime - 86400
        Write-Verbose "Defaulting to startTime 24-hours ago."
    }

    #Formater headers for autentisering
    $bearerAuth = "Bearer $bearerToken"
    $headers = @{ Authorization = $bearerAuth
                  Accept = "application/json"}

    #Formater URL for HTTP-requesten, og henter enheter
    $reqURL = $CPPMURL + "/api/insight/endpoint/time-range/$startTime/$unixTime"

    $CPPMRawData = $null

    Try{
        $CPPMRawData = Invoke-RestMethod -Uri $reqURL -Method Get -Headers $headers -ContentType application/json
        Write-Verbose "Uthenting suksessfull" 
    }Catch{
        $errorMessage = $_.Exception.Message
        Write-Host "Uthenting feilet med følgende feilmelding: $errorMessage" -ForegroundColor Red
    }
    $CPPMRawData._embedded.items
}