<#
    .SYNOPSIS
    Scriptet henter nodedata fra Clearpass, Airwave og SCCM, og konsoliderer det til en enhetstabell. Deretter blir uthentingene sendt til databasen, og mellomlageret oppdateres for analyse.

    .DESCRIPTION
    Scriptet henter først nødvendige funksjoner fra \uthenting og \universalt, og parser deretter konfigurasjonsfilen. Dersom konfigurasjonsfilens allowedTimeFrame for SCCM, AMP og CPPM sier at scriptet får lov, vil scriptet hente ut nodeinformasjonen. For SCCM benyttes kerberos-authentication fra brukeren på maskinen som kjører scriptet, og for AMP og CPPM benyttes APIet. Dersom minimum ett sett med nodedata (fra en av kildene) blir hentet, vil dataen konsolideres, og bli lagret i mellomlageret for analyse. Deretter sendes dataen som ble hentet ut til databasen som er beskrevet i konfigurasjonsfilen.

#>


###################--Dot source av nødvendige funksjoner--###################

$dirs = Get-ChildItem -Path $PSScriptRoot | Where-Object -Property mode -EQ -Value d-----
foreach ($dir in $dirs){
    Get-ChildItem -Path $dir.FullName -Filter *.ps1 | ForEach-Object {. $_.FullName}
}

###################--Parsing av configfil--###################

try {
    $config = Get-ConfigFil -pathTilFil $PSScriptRoot/configuration.ini
}
catch {
    throw "Klarte ikke parse configfil. Sjekk at filen `$PSScriptRoot/configuration.ini finnes, og at den er riktig formatert"

}

$rootPath = $PSScriptRoot

#Universal
$pathTilMellomLager = ""
if($config.RelPaths -eq $true){
    $pathTilMellomLager = $rootPath + $config.pathTilMellomLager
}else{
    $pathTilMellomLager = $config.pathTilMellomLager
}
$SecretMode = $config.SecretMode
$MellomLagerSize = $config.MellomLagerSize

#Clearpass
$CPPMClientId = $config.CPPM_APIClientID
$CPPMURL = $config.CPPM_URL
$CPPMTimeFrame = $config.CPPM_allowedTimeFrame
$CPPMClientSecret = ""
if ($secretMode -eq "plain") {
    $CPPMClientSecret = $config.CPPM_APIClientSecret
}elseif ($SecretMode -eq "file") {
    $CPPMClientSecret = Get-Content $config.CPPM_APIClientSecretPath | Unprotect-Passord
}


#Airwave
$AMPAPIUser = $config.AMP_APIUserName
$AMPReportId = $config.AMP_ReportID
$AMPURL = $config.AMP_URL
$AMPTimeFrame = $config.CPPM_allowedTimeFrame
$AMPAPIpWord = ""
if ($secretMode -eq "plain") {
    $AMPAPIpWord = $config.AMP_APIpWord
}elseif ($SecretMode -eq "file") {
    $AMPAPIpWord = Get-Content $config.AMP_APIpWordPath | Unprotect-Passord
}

#SCCM
$SCCMURL = $config.SCCM_URL
$SCCMSitecode = $config.SCCM_SiteCode
$SCCMQuery = $config.SCCM_Query
$SCCMTimeFrame = $config.SCCM_allowedTimeFrame

#Azure
$AzureUserName = $config.Azure_UserName
$AzureSQLServer = $config.Azure_SQLServer
$AzurePort = $config.Azure_Port
$AzureDatabaseNavn = $config.Azure_databaseNavn

$Azurepassword = ""
if ($secretMode -eq "plain") {
    $Azurepassword = $config.Azure_password
}elseif ($SecretMode -eq "file") {
    $Azurepassword = Get-Content $config.Azure_passwordPath | Unprotect-Passord
}

#SQL
$SQLBatchSize = $config.SQL_BatchSize

<#
if($SQLBatchSize -gt 1000){
    $SQLBatchSize = 1000
}elseif($SQLBatchSize -lt 1){
    $SQLBatchSize = 1
}
#>

###################--Sertifikater for API--##################
try{
add-type @"
using System.Net;
using System.Security.Cryptography.X509Certificates;
public class TrustAllCertsPolicy : ICertificatePolicy {
    public bool CheckValidationResult(
        ServicePoint srvPoint, X509Certificate certificate,
        WebRequest request, int certificateProblem) {
        return true;
    }
}
"@
$AllProtocols = [System.Net.SecurityProtocolType]'Ssl3,Tls,Tls11,Tls12'
[System.Net.ServicePointManager]::SecurityProtocol = $AllProtocols
[System.Net.ServicePointManager]::CertificatePolicy = New-Object TrustAllCertsPolicy
}catch{
    Write-Host "Feil med sertifikater for API"
}

######################--Uthenting av enheter--########################

Write-Debug "Setter Hentet-variabler til FALSE"
$AMPHentet = $false
$SCCMHentet = $false
$CPPMHentet = $false
#Airwave
if (IsInAllottedTime $AMPTimeFrame.split("-")[0] $AMPTimeFrame.split("-")[1]) {
    Write-Verbose "AllowedTimeFrame tillater AMP-uthenting"

    Write-Verbose "Henter ut AirWave rapport $APMReportID fra $AMPURL"
    #$AMPraw = Get-AMPClients -uName $AMPAPIUser -pWord $AMPAPIpWord -airWaveHost $AMPURL -reportID $AMPReportID
    [xml]$AMPraw = Get-Content -Path C:\Users\sccmtest\Desktop\Bachelor-Marius\Eksempeldata\Airwave_raadata.txt

    $ampUseful = $AMPraw.report.pickled_user_inventory

    Write-Verbose "Formaterer innholdet i rapportens .report.pickled_user_inventory"
    $AMPUthenting = Format-AMPClients -AMPData $ampUseful

    Write-Debug "Setter AMPHentet til TRUE"
    $AMPHentet = $true
}else{
    Write-Verbose "AllowedTimeFrame tillater IKKE AMP-uthenting, utfører ingen henting"
}

#Clearpass
if (IsInAllottedTime $CPPMTimeFrame.split("-")[0] $CPPMTimeFrame.split("-")[1]) {
    Write-Verbose "AllowedTimeFrame tillater CPPM-uthenting"

    Write-Verbose "Henter ut og formaterer CPPM-enheter fra $CPPMURL"
    $CPPMUthenting = Get-CPPMApiAccessToken -CPPMURL $CPPMURL -clientId $CPPMClientId -clientSecret $CPPMClientSecret | Get-CPPMEndpoints -CPPMURL $CPPMURL -startTime 0 | Format-CPPMEndpoints

    Write-Debug "Setter CPPMHentet til TRUE"
    $CPPMHentet = $true
}else{
    Write-Verbose "AllowedTimeFrame tillater IKKE CPPM-uthenting, utfører ingen henting"
}

#SCCM
if (IsInAllottedTime $SCCMTimeFrame.split("-")[0] $SCCMTimeFrame.split("-")[1]) {
    Write-Verbose "AllowedTimeFrame tillater SCCM-uthenting"

    Write-Verbose "Henter ut SCCM-query $SCCMQuery fra $SCCMURL"
    $SCCMRAW = Get-SCCMKlienter -query $SCCMQuery -SCCM_URL $SCCMURL -SiteCode $SCCMSiteCode

    Write-Verbose "Formaterer SCCM-klienter"
    $SCCMUthenting = Formater-SCCMData -sccmUthenting $SCCMRAW

    Write-Debug "Setter SCCMHentet til TRUE"
    $SCCMHentet = $true
}else{
    Write-Verbose "AllowedTimeFrame tillater IKKE SCCM-uthenting"
}
#Legg til uthenting av nye datakilder her



###############--Samling og mellomlagring av uthentinger--################
Write-Debug "Initialiserer tempNodeCollection med uthentinger, og nodeDatacollection som tom tabell"

##########--Legg til ferdig formatert informasjon her--#################
$tempNodeCollection = @($SCCMUthenting, $CPPMUthenting, $AMPUthenting) #
########################################################################

$nodeDatacollection = @()

Write-Debug "Starter foreach uthenting in tempnodecollection"
foreach ($tempUthenting in $tempNodeCollection) {
    if ($null -ne $tempUthenting) {
        Write-Debug "tempUthenting er ikke NULL, legger til tempUthenting i nodeDatacollection"
        $nodeDatacollection += ,$tempUthenting
    }else{
        Write-Debug "tempUthenting er NULL, ingen data ble hentet. legger IKKE tempUthenting inn i nodeDataCollection"
    }
}

if($nodeDatacollection.count -ne 0){
    Write-Verbose "Minimum ett sett med data ble hentet, merger og lagrer i mellomlager"

    $nodeDataCollectionCount = $nodeDatacollection.count

    Write-Verbose "Merger nodedata fra $nodeDataCollectionCount collection(s)"
    $mergedData = Merge-NodeData -nodeData $nodeDatacollection

    [String]$pathpre = Get-Date -Date ([datetime]::Now) -Format "yyyy-MM-dd"
    $pathex = ".uthenting"

    Test-Lagringsplass -Lagringsplass $PSScriptRoot -MappeNavn "MellomLager"

    Write-Verbose "Lagrer merget data til fil: $pathTilMellomLager\$pathpre$pathex"
    $mergedData | ConvertTo-Json | Out-File -FilePath $pathTilMellomLager\$pathpre$pathex

    #Fjerner gammel data fra mellomlager
    Remove-DataEldreEnn1Uke -Sti $pathTilMellomLager -LagerSize $MellomLagerSize
}else{
    Write-Verbose "Ingen data ble hentet, ingen data å merge."
}

###############--Innsending av uthentinger til database--################

If ($nodeDatacollection.count -ne 0) {
    Write-Verbose "Minimum ett sett med data ble hentet, sender data til database"

    Write-Verbose "Tester SQL-connection"
    $isSQL = Test-SqlConnection -brukernavn $AzureUserName -passord $Azurepassword -SQLServer $AzureSQLServer -port $AzurePort -DatabaseNavn $AzureDatabaseNavn
    If($isSQL){
        Write-Debug "Connection til database OK"

        Write-Verbose "Batcher enheter fra mergedData"
        $batchedEnheter = Group-SQLInnsending -udeltTing $mergedData.Keys -batchSize $SQLBatchSize
        
        Write-Debug "Starter ForEach enhetBatch in batchedEnheter"
        ForEach($enhetBatch in $batchedEnheter){
            $EnhetQuery = New-SqlQueryEnheter -Enheter $enhetBatch
            Write-Debug "Query som sendes til database: `n$enhetQuery"

            Write-Debug "Prøver å sende query til tabell Enheter i database"
            try {
                Send-QueryToDatabase -Query $EnhetQuery -brukernavn $AzureUserName -passord $Azurepassword -SQLServer $AzureSQLServer -port $AzurePort -DatabaseNavn $AzureDatabaseNavn    
            }
            catch {
                $ErrorMessage = $_.Exception.Message
                Write-Host "Feil i innsending til tabell Enheter i database" -ForegroundColor Red
                Write-Host $ErrorMessage -ForegroundColor Red
            }             
        }

        if($SCCMHentet){
            Write-Debug "SCCMUthenting er hentet, starter batching og innsending"

            Write-Verbose "Batcher enheter fra SCCMUthenting"
            $batchedSCCM = Group-SQLInnsending -udeltTing $SCCMUthenting -batchSize $SQLBatchSize

            Write-Verbose "Sender batcher inn til database"
            ForEach($SCCMbatch in $batchedSCCM){
                $SCCMQuery = New-SqlQuerySCCM -Innsending $SCCMbatch
                Write-Debug "Query som sendes tl database: `n$SCCMQuery"
                
                Write-Debug "Prøver å sende query til tabell SCCM_Uthenting i database"
                try {
                    Send-QueryToDatabase -Query $SCCMQuery -brukernavn $AzureUserName -passord $Azurepassword -SQLServer $AzureSQLServer -port $AzurePort -DatabaseNavn $AzureDatabaseNavn  
                }
                catch {
                    $ErrorMessage = $_.Exception.Message
                    Write-Host "Feil i innsending til tabell SCCM_Uthenting i database" -ForegroundColor Red
                    Write-Host $ErrorMessage -ForegroundColor Red
                }
            }
        }

        if($AMPHentet){
            Write-Debug "AMPUthenting er hentet, starter batching og innsending"

            Write-Verbose "Batcher enheter fra AMPUthenting"
            $batchedAMP = Group-SQLInnsending -udeltTing $AMPUthenting -batchSize $SQLBatchSize

            Write-Verbose "Sender batcher inn til database"
            ForEach($AMPbatch in $batchedAMP){
                $AMPQuery = New-SqlQueryAMP -Innsending $AMPbatch
                Write-Debug "Query som sendes til database: `n$AMPQuery"

                Write-Debug "Prøver å sende query til tabell AMP_Uthenting i database"
                try {
                    Send-QueryToDatabase -Query $AMPQuery -brukernavn $AzureUserName -passord $Azurepassword -SQLServer $AzureSQLServer -port $AzurePort -DatabaseNavn $AzureDatabaseNavn  
                }
                catch {
                    $ErrorMessage = $_.Exception.Message
                    Write-Host "Feil i innsending til tabell AMP_Uthenting i database" -ForegroundColor Red
                    Write-Host $ErrorMessage -ForegroundColor Red
                }                 
            }
        }

        if($CPPMHentet){
            Write-Debug "CPPMUthenting er hentet, starter batching og innsending"

            Write-Verbose "Batcher enheter fra CPPMUthenting"
            $batchedCPPM = Group-SQLInnsending -udeltTing $CPPMUthenting -batchSize $SQLBatchSize

            Write-Verbose "Sender batcher inn til database"
            ForEach($CPPMbatch in $batchedCPPM){
                $CPPMQuery = New-SqlQueryCPPM -Innsending $CPPMbatch
                Write-Debug "Query som sendes til database: `n$CPPMQuery"

                Write-Debug "Prøver å sende query til tabell CPPM_Uthenting i database"
                try {
                    Send-QueryToDatabase -Query $CPPMQuery -brukernavn $AzureUserName -passord $Azurepassword -SQLServer $AzureSQLServer -port $AzurePort -DatabaseNavn $AzureDatabaseNavn 
                }
                catch {
                    $ErrorMessage = $_.Exception.Message
                    Write-Host "Feil i innsending til tabell CPPM_Uthenting i database" -ForegroundColor Red
                    Write-Host $ErrorMessage -ForegroundColor Red
                }                  
            }
        }
    }Else{
        Write-Host "Klarte ikke koble til SQL server. Se [Azure Database] i konfigfil" -ForegroundColor Red
    }
}Else{
    Write-Verbose "Ingen data ble hentet, ingen data å sende til database"
}

